import { Global, Module } from '@nestjs/common';
import { TypeOrmModule } from '@nestjs/typeorm';
import { DataSource } from 'typeorm';
import { DatabaseConfig, databaseConfig } from './database.config';
import { APP_DATASOURCES } from './database.constants';
import { typeormTransactionalDataSourceFactory } from './utils/typeorm-data-source-factory';
import { typeormEntitiesFromImport } from './utils/typeorm-import-entities';
import { typeormModuleOptionsFactory } from './utils/typeorm-module-options-factory';
import { ConfigModule } from '@nestjs/config';

const dataSources: DataSource[] = [];

const typeormModules = [
  TypeOrmModule.forRootAsync({
    imports: databaseConfig.asProvider().imports,
    dataSourceFactory: typeormTransactionalDataSourceFactory(),
    useFactory: async (dbConfig: DatabaseConfig) => {
      // const entities = await import('./path/to/entities');
      const entities = {};

      const config = await typeormModuleOptionsFactory(dbConfig, [typeormEntitiesFromImport(entities)]);
      return config;
    },
    inject: [databaseConfig.KEY],
  }),
];

const dataSourcesProvider = {
  provide: APP_DATASOURCES,
  useValue: dataSources,
};

@Global()
@Module({
  imports: [ConfigModule.forFeature(databaseConfig), ...typeormModules],
  providers: [dataSourcesProvider],
  exports: [...typeormModules, dataSourcesProvider],
})
export class AppDatabaseModule {}
